@@ -1,6 +1,6 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
-// Copyright Bob Friesenhahn, 1999 - 2014
+// Copyright Bob Friesenhahn, 1999 - 2015
 //
 // Inclusion of GraphicsMagick headers (with namespace magic)
 
@@ -129,6 +129,9 @@ namespace MagickLib
 #          pragma comment(lib, "CORE_DB_ttf_.lib")
 #        endif
 #        pragma comment(lib, "CORE_DB_wand_.lib")
+#        if defined(HasWEBP)
+#          pragma comment(lib, "CORE_DB_webp_.lib")
+#        endif
 #        if defined(HasWMFlite)
 #          pragma comment(lib, "CORE_DB_wmf_.lib")
 #        endif
@@ -171,6 +174,9 @@ namespace MagickLib
 #          pragma comment(lib, "CORE_RL_ttf_.lib")
 #        endif
 #        pragma comment(lib, "CORE_RL_wand_.lib")
+#        if defined(HasWEBP)
+#          pragma comment(lib, "CORE_RL_webp_.lib")
+#        endif
 #        if defined(HasWMFlite)
 #          pragma comment(lib, "CORE_RL_wmf_.lib")
 #        endif
@@ -218,7 +224,7 @@ namespace Magick
   using MagickLib::UndefinedClass;
   using MagickLib::DirectClass;
   using MagickLib::PseudoClass;
-  
+
   // Channel types
   using MagickLib::ChannelType;
   using MagickLib::UndefinedChannel;
@@ -233,7 +239,7 @@ namespace Magick
   using MagickLib::MatteChannel;
   using MagickLib::AllChannels;
   using MagickLib::GrayChannel;
-  
+
   // Color-space types
   using MagickLib::ColorspaceType;
   using MagickLib::UndefinedColorspace;
@@ -256,7 +262,7 @@ namespace Magick
   using MagickLib::Rec601LumaColorspace;
   using MagickLib::Rec709LumaColorspace;
   using MagickLib::Rec709YCbCrColorspace;
-  
+
   // Composition operations
   using MagickLib::AddCompositeOp;
   using MagickLib::AtopCompositeOp;
@@ -307,7 +313,7 @@ namespace Magick
   using MagickLib::VividLightCompositeOp;
   using MagickLib::PinLightCompositeOp;
   using MagickLib::HardMixCompositeOp;
-  
+
   // Compression algorithms
   using MagickLib::CompressionType;
   using MagickLib::UndefinedCompression;
@@ -467,7 +473,7 @@ namespace Magick
   using MagickLib::UndefinedRule;
   using MagickLib::EvenOddRule;
   using MagickLib::NonZeroRule;
-  
+
   // Filter types
   using MagickLib::FilterTypes;
   using MagickLib::UndefinedFilter;
@@ -514,7 +520,7 @@ namespace Magick
   using MagickLib::ColorSeparationType;
   using MagickLib::ColorSeparationMatteType;
   using MagickLib::OptimizeType;
-  
+
   // Interlace types
   using MagickLib::InterlaceType;
   using MagickLib::UndefinedInterlace;
@@ -558,7 +564,7 @@ namespace Magick
   using MagickLib::RightTopOrientation;
   using MagickLib::RightBottomOrientation;
   using MagickLib::LeftBottomOrientation;
-  
+
   // Paint methods
   using MagickLib::PaintMethod;
   using MagickLib::PointMethod;
@@ -665,7 +671,7 @@ namespace Magick
   using MagickLib::PerceptualIntent;
   using MagickLib::AbsoluteIntent;
   using MagickLib::RelativeIntent;
-  
+
   // Resolution units
   using MagickLib::ResolutionType;
   using MagickLib::UndefinedResolution;
@@ -725,6 +731,8 @@ namespace Magick
   using MagickLib::MemoryResource;
   using MagickLib::PixelsResource;
   using MagickLib::ThreadsResource;
+  using MagickLib::WidthResource;
+  using MagickLib::HeightResource;
 
   // Virtual pixel methods
   using MagickLib::VirtualPixelMethod;
@@ -905,6 +913,7 @@ namespace Magick
   using MagickLib::ExceptionInfo;
   using MagickLib::ExecuteModuleProcess;
   using MagickLib::ExportImagePixelArea;
+  using MagickLib::ExtentImage;
   using MagickLib::FileOpenError;
   using MagickLib::FileOpenFatalError;
   using MagickLib::FileOpenWarning;
@@ -930,6 +939,7 @@ namespace Magick
   using MagickLib::GetImageChannelDepth;
   using MagickLib::GetImageClipMask;
   using MagickLib::GetImageDepth;
+  using MagickLib::GetImageGeometry;
   using MagickLib::GetImageInfo;
   using MagickLib::GetImagePixels;
   using MagickLib::GetImageProfile;
@@ -973,6 +983,7 @@ namespace Magick
   using MagickLib::MagickInfo;
   using MagickLib::MagickMalloc;
   using MagickLib::MagickRealloc;
+  using MagickLib::MagickStrlCpy;
   using MagickLib::MagickToMime;
   using MagickLib::MagnifyImage;
   using MagickLib::MapImage;
@@ -1028,6 +1039,7 @@ namespace Magick
   using MagickLib::RegistryType;
   using MagickLib::RegistryWarning;
   using MagickLib::RemoveDefinitions;
+  using MagickLib::ResizeImage;
   using MagickLib::ResourceLimitError;
   using MagickLib::ResourceLimitFatalError;
   using MagickLib::ResourceLimitWarning;
