@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2015 GraphicsMagick Group
+% Copyright (C) 2003-2017 GraphicsMagick Group
 %
 % This program is covered by multiple licenses, which are described in
 % Copyright.txt. You should have received a copy of Copyright.txt with this
@@ -130,7 +130,7 @@ typedef struct _CineonImageInfo
   U8    sense;                     /* Image sense: 0=positive, 1=negative */
   U32   eol_pad;                   /* End of line padding */
   U32   eoc_pad;                   /* End of channel padding */
-  ASCII reserved2[20];             /* Reserved for future use */  
+  ASCII reserved2[20];             /* Reserved for future use */
 } CineonImageInfo;
 
 typedef struct _CineonImageOriginationInfo
@@ -348,7 +348,7 @@ static void SwabCineonImageInfo(CineonImageInfo *image_info)
   MagickSwabFloat(&image_info->blue_primary_chromaticity[0].f);
   MagickSwabFloat(&image_info->blue_primary_chromaticity[1].f);
   MagickSwabUInt32(&image_info->eol_pad);
-  MagickSwabUInt32(&image_info->eoc_pad);  
+  MagickSwabUInt32(&image_info->eoc_pad);
 }
 
 static void SwabCineonImageOriginationInfo(CineonImageOriginationInfo *image_info)
@@ -472,14 +472,14 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
     i;
 
   size_t
-    offset;    
+    offset;
 
   unsigned int
     channel,
     max_bits_per_sample,
     max_lines_per_image,
     max_pixels_per_line,
-    number_of_channels;    
+    number_of_channels;
 
   MagickPassFail
     status;
@@ -492,7 +492,7 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
 
   unsigned char
     *scandata;
-  
+
   void
     *scanline;
 
@@ -614,7 +614,7 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
       max_lines_per_image=Max(max_lines_per_image,
                               cin_image_info.channel_info[channel].lines_per_image);
     }
-  
+
   image->depth=max_bits_per_sample;
   image->columns=max_pixels_per_line;
   image->rows=max_lines_per_image;
@@ -705,14 +705,14 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
     {
       unsigned char
         *user_data;
-      
+
       const size_t
         block_size = 65536UL;
-      
+
       size_t
         read_size,
         user_data_length;
-      
+
       user_data_length=0UL;
       user_data=(unsigned char *) NULL;
       while (user_data_length < cin_file_info.user_defined_length)
@@ -774,6 +774,8 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
           scandata_bytes=4;
           scale_to_short=64;
           scandata=MagickAllocateMemory(unsigned char *,scandata_bytes);
+          if (scandata == (unsigned char *) NULL)
+            ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
           scanline=scandata;
           MagickBitStreamInitializeRead(&bit_stream,scanline);
           for (y=0; y < (long) image->rows; y++)
@@ -808,7 +810,7 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,exception,
                                               LoadImageText,image->filename,
-					      image->columns,image->rows))
+                                              image->columns,image->rows))
                     break;
             }
           MagickFreeMemory(scandata);
@@ -816,9 +818,11 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
         }
       case 3:
         {
-          scandata_bytes=image->columns*4;
+          scandata_bytes=MagickArraySize(image->columns,4);
           scale_to_short=64;
           scandata=MagickAllocateMemory(unsigned char *,scandata_bytes);
+          if (scandata == (unsigned char *) NULL)
+            ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
           for (y=0; y < (long) image->rows; y++)
             {
               magick_uint32_t red;
@@ -857,7 +861,7 @@ static Image *ReadCINEONImage(const ImageInfo *image_info,
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,exception,
                                               LoadImageText,image->filename,
-					      image->columns,image->rows))
+                                              image->columns,image->rows))
                     break;
             }
           MagickFreeMemory(scandata);
@@ -1125,7 +1129,7 @@ static unsigned int WriteCINEONImage(const ImageInfo *image_info,Image *image)
 
   register const PixelPacket
     *p;
-  
+
   register long
     x;
 
@@ -1263,7 +1267,7 @@ static unsigned int WriteCINEONImage(const ImageInfo *image_info,Image *image)
   cin_image_info.eol_pad = 0;
   /* End of channel padding */
   cin_image_info.eoc_pad = 0;
-  
+
   /*
     Image origination header.
   */
@@ -1328,7 +1332,7 @@ static unsigned int WriteCINEONImage(const ImageInfo *image_info,Image *image)
   AttributeToString(image_info,image,"DPX:mp.frame.id",cin_mp_info.frame_id);
   /* Slate information */
   AttributeToString(image_info,image,"DPX:mp.slate.info",cin_mp_info.slate_info);
-  
+
   /*
     Open output image file.
   */
@@ -1431,7 +1435,7 @@ static unsigned int WriteCINEONImage(const ImageInfo *image_info,Image *image)
     bits_per_sample = cin_image_info.channel_info[0].bits_per_sample;
     scale_from_short=(65535U / (65535U >> (16-bits_per_sample)));
 
-    scanline_bytes=image->columns*4;
+    scanline_bytes=MagickArraySize(image->columns,4);
     scanline=MagickAllocateMemory(unsigned char *,scanline_bytes);
     if (scanline == (unsigned char *) NULL)
       ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
@@ -1442,9 +1446,9 @@ static unsigned int WriteCINEONImage(const ImageInfo *image_info,Image *image)
         p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
-        
+
         MagickBitStreamInitializeWrite(&bit_stream,scanline);
-        
+
         for (x=0; x < (long) image->columns; x++)
           {
             red   = ScaleQuantumToShort(p->red)/scale_from_short;
@@ -1472,7 +1476,7 @@ static unsigned int WriteCINEONImage(const ImageInfo *image_info,Image *image)
           if (QuantumTick(y,image->rows))
             if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                         SaveImageText,image->filename,
-					image->columns,image->rows))
+                                        image->columns,image->rows))
               break;
       }
     MagickFreeMemory(scanline);
