@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2015 GraphicsMagick Group
+% Copyright (C) 2003 - 2017 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -314,14 +314,14 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     char
       options[MaxTextExtent],
       arg[MaxTextExtent];
-    
+
     options[0]='\0';
     /*
       Append subrange.
     */
     if (image_info->subrange != 0)
       FormatString(options,"-dFirstPage=%lu -dLastPage=%lu",
-		   image_info->subimage+1,image_info->subimage+image_info->subrange);
+                   image_info->subimage+1,image_info->subimage+image_info->subrange);
     /*
       Append bounding box.
     */
@@ -335,14 +335,14 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (void) LiberateTemporaryFile((char *) image_info->filename);
     if(!AcquireTemporaryFileName((char *) image_info->filename))
       {
-	(void) LiberateTemporaryFile(postscript_filename);
-	ThrowReaderTemporaryFileException(image_info->filename);
+        (void) LiberateTemporaryFile(postscript_filename);
+        ThrowReaderTemporaryFileException(image_info->filename);
       }
     FormatString(command,delegate_info->commands,antialias,
-		 antialias,density,options,image_info->filename,
-		 postscript_filename);
+                 antialias,density,options,image_info->filename,
+                 postscript_filename);
   }
-  (void) MagickMonitorFormatted(0,8,&image->exception,RenderPostscriptText,
+  (void) MagickMonitorFormatted(0,8,exception,RenderPostscriptText,
                                 image_info->filename);
   status=InvokePostscriptDelegate(image_info->verbose,command,exception);
   if (!IsAccessibleAndNotEmpty(image_info->filename))
@@ -361,15 +361,15 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       status=InvokePostscriptDelegate(image_info->verbose,command,exception);
     }
   (void) LiberateTemporaryFile(postscript_filename);
-  (void) MagickMonitorFormatted(7,8,&image->exception,RenderPostscriptText,
+  (void) MagickMonitorFormatted(7,8,exception,RenderPostscriptText,
                                 image_info->filename);
   if (IsAccessibleAndNotEmpty(image_info->filename))
     {
       /*
-	Read Ghostscript output.
+        Read Ghostscript output.
       */
       ImageInfo
-	*clone_info;
+        *clone_info;
 
       clone_info=CloneImageInfo(image_info);
       clone_info->blob=(void *) NULL;
@@ -395,25 +395,25 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (image == (Image *) NULL)
     {
       if (UndefinedException == exception->severity)
-	ThrowException(exception,DelegateError,PostscriptDelegateFailed,filename);
+        ThrowException(exception,DelegateError,PostscriptDelegateFailed,filename);
     }
   else
     {
       do
-	{
-	  (void) strlcpy(image->magick,"PS",sizeof(image->magick));
-	  (void) strlcpy(image->filename,filename,sizeof(image->filename));
-	  next_image=SyncNextImageInList(image);
-	  if (next_image != (Image *) NULL)
-	    image=next_image;
-	} while (next_image != (Image *) NULL);
+        {
+          (void) strlcpy(image->magick,"PS",sizeof(image->magick));
+          (void) strlcpy(image->filename,filename,sizeof(image->filename));
+          next_image=SyncNextImageInList(image);
+          if (next_image != (Image *) NULL)
+            image=next_image;
+        } while (next_image != (Image *) NULL);
       while (image->previous != (Image *) NULL)
-	image=image->previous;
+        image=image->previous;
       if (image_info->subimage != 0)
         {
           unsigned long
             scene = image_info->subimage;
-          
+
           for (next_image=image;
                next_image != (Image *) NULL;
                next_image=next_image->next)
@@ -1302,9 +1302,9 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                               SaveImageText,image->filename,
-					      image->columns,image->rows))
+                                              image->columns,image->rows))
                     break;
-            } 
+            }
             if (bp != buffer)
               {
                 *bp++='\n';
@@ -1372,7 +1372,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                 if (QuantumTick(y,image->rows))
                   if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                               SaveImageText,image->filename,
-					      image->columns,image->rows))
+                                              image->columns,image->rows))
                     break;
             }
             if (bp != buffer)
@@ -1439,7 +1439,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                   if (QuantumTick(y,image->rows))
                     if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                                 SaveImageText,image->filename,
-						image->columns,image->rows))
+                                                image->columns,image->rows))
                       break;
               }
               if (bp != buffer)
@@ -1486,7 +1486,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                   if (QuantumTick(y,image->rows))
                     if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                                 SaveImageText,image->filename,
-						image->columns,image->rows))
+                                                image->columns,image->rows))
                       break;
               }
               if (bp != buffer)
@@ -1572,7 +1572,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                   if (QuantumTick(y,image->rows))
                     if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                                 SaveImageText,image->filename,
-						image->columns,image->rows))
+                                                image->columns,image->rows))
                       break;
               }
               if (bp != buffer)
@@ -1614,7 +1614,7 @@ static unsigned int WritePSImage(const ImageInfo *image_info,Image *image)
                   if (QuantumTick(y,image->rows))
                     if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                                 SaveImageText,image->filename,
-						image->columns,image->rows))
+                                                image->columns,image->rows))
                       break;
               }
               if (bp != buffer)
