@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2015 GraphicsMagick Group
+% Copyright (C) 2003-2017 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -162,13 +162,13 @@ static unsigned int IsSUN(const unsigned char *magick,const size_t length)
 %
 %  The format of the DecodeImage method is:
 %
-%      unsigned int DecodeImage(const unsigned char *compressed_pixels,
+%      MagickPassFail DecodeImage(const unsigned char *compressed_pixels,
 %        const size_t length,unsigned char *pixels)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method DecodeImage returns True if all the pixels are
-%      uncompressed without error, otherwise False.
+%    o status:  Method DecodeImage returns True (MagickPass) if all the
+%      pixels are uncompressed without error, otherwise False (MagickFail).
 %
 %    o compressed_pixels:  The address of a byte (8 bits) array of compressed
 %      pixel data.
@@ -209,26 +209,37 @@ DecodeImage(const unsigned char *compressed_pixels,
   q=pixels;
   while (((size_t) (p-compressed_pixels) < compressed_size) &&
          ((size_t) (q-pixels) < pixels_size))
-  {
-    byte=(*p++);
-    if (byte != 128U)
-      *q++=byte;
-    else
-      {
-        /*
-          Runlength-encoded packet: <count><byte>
-        */
-        count=(*p++);
-        if (count > 0)
-          byte=(*p++);
-        while ((count >= 0) && ((size_t) (q-pixels) < pixels_size))
+    {
+      byte=(*p++);
+      if (byte != 128U)
         {
+          /*
+            Stand-alone byte
+          */
           *q++=byte;
-          count--;
         }
-     }
-  }
-  return (((size_t) (q-pixels) == pixels_size) ? MagickTrue : MagickFalse);
+      else
+        {
+          /*
+            Runlength-encoded packet: <count><byte>
+          */
+          if (((size_t) (p-compressed_pixels) >= compressed_size))
+            break;
+          count=(*p++);
+          if (count > 0)
+            {
+              if (((size_t) (p-compressed_pixels) >= compressed_size))
+                break;
+              byte=(*p++);
+            }
+          while ((count >= 0) && ((size_t) (q-pixels) < pixels_size))
+            {
+              *q++=byte;
+              count--;
+            }
+        }
+    }
+  return (((size_t) (q-pixels) == pixels_size) ? MagickPass : MagickFail);
 }
 
 /*
@@ -479,7 +490,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
       }
       default:
         ThrowReaderException(CoderError,ColormapTypeNotSupported,image)
-    } 
+    }
     image->matte=(sun_info.depth == 32);
     image->columns=sun_info.width;
     image->rows=sun_info.height;
@@ -487,6 +498,12 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (sun_info.depth < 8)
       image->depth=sun_info.depth;
 
+    if (image_info->ping)
+      {
+        CloseBlob(image);
+        return(image);
+      }
+
     /*
       Compute bytes per line and bytes per image for an unencoded
       image.
@@ -511,15 +528,37 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (bytes_per_image > sun_info.length)
         ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
 
-    if (image_info->ping)
-      {
-        CloseBlob(image);
-        return(image);
-      }
     if (sun_info.type == RT_ENCODED)
       sun_data_length=(size_t) sun_info.length;
     else
       sun_data_length=bytes_per_image;
+
+    /*
+      Verify that data length claimed by header is supported by file size
+    */
+    if (sun_info.type == RT_ENCODED)
+      {
+        if (sun_data_length < bytes_per_image/255U)
+          {
+            ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+          }
+      }
+    if (BlobIsSeekable(image))
+      {
+        const magick_off_t file_size = GetBlobSize(image);
+        const magick_off_t current_offset = TellBlob(image);
+        if ((file_size > 0) &&
+            (current_offset > 0) &&
+            (file_size > current_offset))
+        {
+          const magick_off_t remaining = file_size-current_offset;
+          if (remaining < (magick_off_t) sun_data_length)
+            {
+              ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image);
+            }
+        }
+      }
+
     sun_data=MagickAllocateMemory(unsigned char *,sun_data_length);
     if (sun_data == (unsigned char *) NULL)
       ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
@@ -537,8 +576,11 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         sun_pixels=MagickAllocateMemory(unsigned char *,bytes_per_image);
         if (sun_pixels == (unsigned char *) NULL)
-          ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
-            image);
+          {
+            MagickFreeMemory(sun_data);
+            ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
+                                 image);
+          }
         status &= DecodeImage(sun_data,sun_data_length,sun_pixels,bytes_per_image);
         MagickFreeMemory(sun_data);
         if (status != MagickPass)
@@ -566,6 +608,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
           for (bit=7; bit >= 0; bit--)
             {
               index=((*p) & (0x01 << bit) ? 0x01 : 0x00);
+              VerifyColormapIndex(image,index);
               indexes[x+7-bit]=index;
               q[x+7-bit]=image->colormap[index];
             }
@@ -576,6 +619,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
             for (bit=7; bit >= (long) (8-(image->columns % 8)); bit--)
               {
                 index=((*p) & (0x01 << bit) ? 0x01 : 0x00);
+                VerifyColormapIndex(image,index);
                 indexes[x+7-bit]=index;
                 q[x+7-bit]=image->colormap[index];
               }
@@ -589,7 +633,7 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
           if (QuantumTick(y,image->rows))
             if (!MagickMonitorFormatted(y,image->rows,exception,
                                         LoadImageText,image->filename,
-					image->columns,image->rows))
+                                        image->columns,image->rows))
               break;
       }
     else
@@ -619,10 +663,10 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (QuantumTick(y,image->rows))
                 if (!MagickMonitorFormatted(y,image->rows,exception,
                                             LoadImageText,image->filename,
-					    image->columns,image->rows))
+                                            image->columns,image->rows))
                   break;
           }
-	}
+        }
       else
         {
           /*
@@ -665,10 +709,10 @@ static Image *ReadSUNImage(const ImageInfo *image_info,ExceptionInfo *exception)
               if (QuantumTick(y,image->rows))
                 if (!MagickMonitorFormatted(y,image->rows,exception,
                                             LoadImageText,image->filename,
-					    image->columns,image->rows))
+                                            image->columns,image->rows))
                   break;
           }
-	}
+        }
     MagickFreeMemory(sun_pixels);
     if (EOFBlob(image))
       {
@@ -927,15 +971,19 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
           *q;
 
         size_t
-          length;
+          length,
+          pad;
 
         unsigned char
           *pixels;
 
         /*
           Allocate memory for pixels.
+
+          Scanlines are padded to 16-bit boundary so account for padding.
         */
-        length=image->columns*sizeof(PixelPacket);
+        pad=(image->columns & 0x01 ? 1 : 0);
+        length=(image->columns + pad) *sizeof(PixelPacket);
         pixels=MagickAllocateMemory(unsigned char *,length);
         if (pixels == (unsigned char *) NULL)
           ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
@@ -965,7 +1013,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                           SaveImageText,image->filename,
-					  image->columns,image->rows))
+                                          image->columns,image->rows))
                 break;
         }
         MagickFreeMemory(pixels);
@@ -1017,7 +1065,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
               if (QuantumTick(y,image->rows))
                 if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                             SaveImageText,image->filename,
-					    image->columns,image->rows))
+                                            image->columns,image->rows))
                   break;
           }
         }
@@ -1052,7 +1100,7 @@ static unsigned int WriteSUNImage(const ImageInfo *image_info,Image *image)
               if (QuantumTick(y,image->rows))
                 if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                             SaveImageText,image->filename,
-					    image->columns,image->rows))
+                                            image->columns,image->rows))
                   break;
           }
         }
