@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2015 GraphicsMagick Group
+% Copyright (C) 2003-2017 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -84,6 +84,8 @@ static unsigned int
 #define ThrowYUVReaderException(code_,reason_,image_) \
 { \
   MagickFreeMemory(scanline);                 \
+  DestroyImage(chroma_image);                 \
+  DestroyImage(resize_image);                 \
   ThrowReaderException(code_,reason_,image_); \
 }
 
@@ -274,7 +276,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (image->previous == (Image *) NULL)
         if (!MagickMonitorFormatted(y,image->rows,exception,LoadImageText,
                                     image->filename,
-				    image->columns,image->rows))
+                                    image->columns,image->rows))
           {
             status=MagickFail;
             break;
@@ -355,6 +357,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     resize_image=ResizeImage(chroma_image,image->columns,image->rows,
       TriangleFilter,1.0,exception);
     DestroyImage(chroma_image);
+    chroma_image=(Image *) NULL;
     if (resize_image == (Image *) NULL)
       ThrowYUVReaderException(ResourceLimitError,MemoryAllocationFailed,image);
     for (y=0; y < (long) image->rows; y++)
@@ -381,6 +384,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
     }
     DestroyImage(resize_image);
+    resize_image=(Image *) NULL;
     if (status == MagickFail)
       break;
     image->colorspace=YCbCrColorspace;
@@ -412,6 +416,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (image->next == (Image *) NULL)
           {
             DestroyImageList(image);
+            MagickFreeMemory(scanline);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
@@ -648,7 +653,7 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                           SaveImageText,image->filename,
-					  image->columns,image->rows))
+                                          image->columns,image->rows))
                 break;
         }
         DestroyImage(yuv_image);
@@ -673,7 +678,7 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
                                           SaveImageText,image->filename,
-					  image->columns,image->rows))
+                                          image->columns,image->rows))
                 break;
         }
         DestroyImage(yuv_image);
