@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2012 GraphicsMagick Group
+% Copyright (C) 2003 - 2017 GraphicsMagick Group
 % Copyright (C) 2003 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -182,7 +182,7 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
           (void) fprintf(file," (%s pixels/s)",format);
         }
       (void) fprintf(file,"\n");
-    
+
       return (ferror(file) ? MagickFail : MagickPass);
     }
   /*
@@ -434,10 +434,14 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
       char
         tuple[MaxTextExtent];
 
+      MagickBool
+        found_transparency;
+
       register const PixelPacket
         *p;
 
       p=(PixelPacket *) NULL;
+      found_transparency = MagickFalse;
       for (y=0; y < image->rows; y++)
         {
           p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
@@ -446,13 +450,16 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
           for (x=0; x < image->columns; x++)
             {
               if (p->opacity == TransparentOpacity)
-                break;
+                {
+                  found_transparency=MagickTrue;
+                  break;
+                }
               p++;
             }
           if (x < image->columns)
             break;
         }
-      if ((x < image->columns) || (y < image->rows))
+      if (found_transparency)
         {
           GetColorTuple(p,image->depth,image->matte,False,tuple);
           (void) fprintf(file,"  Opacity: %.1024s\t",tuple);
@@ -576,8 +583,8 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
   FormatSize(GetBlobSize(image),format);
   (void) fprintf(file,"  Filesize: %.1024s\n",format);
   fprintf(file,"  Interlace: %s\n",
-	  InterlaceTypeToString(image->interlace == UndefinedInterlace ?
-				NoInterlace : image->interlace));
+          InterlaceTypeToString(image->interlace == UndefinedInterlace ?
+                                NoInterlace : image->interlace));
   (void) fprintf(file,"  Orientation: %s\n",
                  OrientationTypeToString(image->orientation));
   (void) QueryColorname(image,&image->background_color,SVGCompliance,color,
@@ -593,7 +600,7 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
     (void) fprintf(file,"  Page geometry: %lux%lu%+ld%+ld\n",image->page.width,
                    image->page.height,image->page.x,image->page.y);
   (void) fprintf(file,"  Compose: %s\n",
-		 CompositeOperatorToString(image->compose));
+                 CompositeOperatorToString(image->compose));
   (void) fprintf(file,"  Dispose: ");
   switch (image->dispose)
     {
@@ -618,7 +625,7 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
     if (image->scene != 0)
       (void) fprintf(file,"  Scene: %lu\n",image->scene);
   (void) fprintf(file,"  Compression: %s\n",
-		 CompressionTypeToString(image->compression));
+                 CompressionTypeToString(image->compression));
   /*
     Display formatted image attributes. This must happen before we access
     any pseudo attributes like EXIF since doing so causes real attributes
@@ -629,15 +636,15 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
     for ( ; attribute != (const ImageAttribute *) NULL;
           attribute=attribute->next)
       {
-	if (LocaleNCompare("EXIF",attribute->key,4) != 0)
-	  {
-	    (void) fprintf(file,"  %c", toupper((int)attribute->key[0]));
-	    if (strlen(attribute->key) > 1)
-	      (void) fprintf(file,"%.1024s",attribute->key+1);
-	    
-	    (void) fprintf(file,": ");
-	    (void) fprintf(file,"%s\n",attribute->value);
-	  }
+        if (LocaleNCompare("EXIF",attribute->key,4) != 0)
+          {
+            (void) fprintf(file,"  %c", toupper((int)attribute->key[0]));
+            if (strlen(attribute->key) > 1)
+              (void) fprintf(file,"%.1024s",attribute->key+1);
+
+            (void) fprintf(file,": ");
+            (void) fprintf(file,"%s\n",attribute->value);
+          }
       }
   }
   if((profile=GetImageProfile(image,"ICM",&profile_length)) != 0)
@@ -657,7 +664,7 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
       */
       (void) fprintf(file,"  Profile-iptc: %lu bytes\n",(unsigned long)
                      profile_length);
-      for (i=0; i < profile_length; )
+      for (i=0; i+5U < profile_length; )
         {
           if (profile[i] != 0x1c)
             {
@@ -685,7 +692,7 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
             case 60: tag=(char *) "Created Time"; break;
             case 65: tag=(char *) "Originating Program"; break;
             case 70: tag=(char *) "Program Version"; break;
-            case 75: tag=(char *) "Object Cycle"; break;
+            case 75: tag=(char *) "Object Cyc"; break;
             case 80: tag=(char *) "Byline"; break;
             case 85: tag=(char *) "Byline Title"; break;
             case 90: tag=(char *) "City"; break;
@@ -726,6 +733,7 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
           (void) fprintf(file,"    %.1024s:\n",tag);
           length=profile[i++] << 8;
           length|=profile[i++];
+          length=Min(length,profile_length-i);
           text=MagickAllocateMemory(char *,length+1);
           if (text != (char *) NULL)
             {
@@ -755,13 +763,13 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
   {
     const char
       *profile_name;
-    
+
     size_t
       profile_length;
-              
+
     const unsigned char *
       profile_info;
-    
+
     ImageProfileIterator
       profile_iterator;
 
@@ -787,10 +795,10 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
               {
                 char
                   **values;
-                
+
                 register char
                   *p;
-                
+
                 values=StringToList(attribute->value);
                 if (values != (char **) NULL)
                   {
@@ -850,7 +858,8 @@ MagickExport MagickPassFail DescribeImage(Image *image,FILE *file,
       for (p=image->directory; *p != '\0'; p++)
         {
           q=p;
-          while ((*q != '\n') && (*q != '\0'))
+          while ((*q != '\n') && (*q != '\0') &&
+                 ((size_t) (q-p) < sizeof(image_info->filename)))
             q++;
           (void) strncpy(image_info->filename,p,q-p);
           image_info->filename[q-p]='\0';
