@@ -351,55 +351,55 @@ ColorMatrixImagePixels(void *mutable_data,         /* User provided mutable data
   for (i=0; i < npixels; i++)
     {
       unsigned int
-	row;
+        row;
 
       /*
-	Accumulate float input pixel
+        Accumulate float input pixel
       */
       column[0]=(double) pixels[i].red;
       column[1]=(double) pixels[i].green;
       column[2]=(double) pixels[i].blue;
       if (image->matte)
-	column[3]=(MaxRGBDouble-(double) pixels[i].opacity);
+        column[3]=(MaxRGBDouble-(double) pixels[i].opacity);
 
       /*
-	Compute row sums.
+        Compute row sums.
       */
       for (row=0; row < 4; row++)
-	{
-	  const double
-	    *m;
+        {
+          const double
+            *m;
 
-	  if ((m = options->matrix[row]) != (const double *) NULL)
-	    sums[row]=m[0]*column[0] + m[1]*column[1] + m[2]*column[2] +
-	      m[3]*column[3] + m[4]*column[4];
-	}
+          if ((m = options->matrix[row]) != (const double *) NULL)
+            sums[row]=m[0]*column[0] + m[1]*column[1] + m[2]*column[2] +
+              m[3]*column[3] + m[4]*column[4];
+        }
 
       /*
-	Assign results.
+        Assign results.
       */
       for (row=0; row < 4; row++)
-	{
-	  if (options->matrix[row] != (const double *) NULL)
-	    {
-	      switch (row)
-		{
-		case 0:
-		  pixels[i].red = RoundDoubleToQuantum(sums[row]);
-		  break;
-		case 1:
-		  pixels[i].green = RoundDoubleToQuantum(sums[row]);
-		  break;
-		case 2:
-		  pixels[i].blue = RoundDoubleToQuantum(sums[row]);
-		  break;
-		case 3:
-		  sums[row]=(MaxRGBDouble-sums[row]);
-		  pixels[i].opacity = RoundDoubleToQuantum(sums[row]);
-		  break;
-		}
-	    }
-	}
+        {
+          if (options->matrix[row] != (const double *) NULL)
+            {
+              switch (row)
+                {
+                case 0:
+                  pixels[i].red = RoundDoubleToQuantum(sums[row]);
+                  break;
+                case 1:
+                  pixels[i].green = RoundDoubleToQuantum(sums[row]);
+                  break;
+                case 2:
+                  pixels[i].blue = RoundDoubleToQuantum(sums[row]);
+                  break;
+                case 3:
+                  sums[row]=(MaxRGBDouble-sums[row]);
+                  pixels[i].opacity = RoundDoubleToQuantum(sums[row]);
+                  break;
+                }
+            }
+        }
     }
 
   return MagickPass;
@@ -433,7 +433,7 @@ ColorMatrixImage(Image *image,const unsigned int order,const double *color_matri
 
   if ((order < 1) || (order > 5))
     ThrowBinaryException(OptionError,MatrixOrderOutOfRange,
-			 MagickMsg(OptionError,UnableToColorMatrixImage));
+                         MagickMsg(OptionError,UnableToColorMatrixImage));
 
   assert(color_matrix != (const double *) NULL);
 
@@ -453,16 +453,16 @@ ColorMatrixImage(Image *image,const unsigned int order,const double *color_matri
     u = color_matrix;
     for (i=0; i < order; i++)
       {
-	d = &matrix[i*5];
-	for (j=0; j < order; j++)
-	  {
-	    if (d[j] != *u)
-	      {
-		d[j]=*u;
-		options.matrix[i]=&matrix[i*5];
-	      }
-	    u++;
-	  }
+        d = &matrix[i*5];
+        for (j=0; j < order; j++)
+          {
+            if (d[j] != *u)
+              {
+                d[j]=*u;
+                options.matrix[i]=&matrix[i*5];
+              }
+            u++;
+          }
       }
 
     /*
@@ -519,18 +519,18 @@ ColorMatrixImage(Image *image,const unsigned int order,const double *color_matri
     {
       image->storage_class=DirectClass;
       /*
-	We don't currently handle CMYK(A) colorspaces, although
-	manipulation in other alternate colorspaces may be useful.
+        We don't currently handle CMYK(A) colorspaces, although
+        manipulation in other alternate colorspaces may be useful.
       */
       if (image->colorspace == CMYKColorspace)
-	(void) TransformColorspace(image,RGBColorspace);
+        (void) TransformColorspace(image,RGBColorspace);
       status=PixelIterateMonoModify(ColorMatrixImagePixels,
-				    NULL,
-				    ColorMatrixImageText,
-				    NULL,&options,
-				    0,0,image->columns,image->rows,
-				    image,
-				    &image->exception);
+                                    NULL,
+                                    ColorMatrixImageText,
+                                    NULL,&options,
+                                    0,0,image->columns,image->rows,
+                                    image,
+                                    &image->exception);
     }
 
   return status;
@@ -688,15 +688,20 @@ MagickExport Image *ImplodeImage(const Image *image,const double amount,
                     factor=1.0;
                     if (distance > 0.0)
                       factor=pow(sin(MagickPI*sqrt(distance)/radius/2),-amount);
-                    InterpolateViewColor(image_view,q,
-                                         factor*x_distance/x_scale+x_center,
-                                         factor*y_distance/y_scale+y_center,
-                                         exception);
+                    if (InterpolateViewColor(image_view,q,
+                                             factor*x_distance/x_scale+x_center,
+                                             factor*y_distance/y_scale+y_center,
+                                             exception) == MagickFail)
+                      {
+                        thread_status=MagickFail;
+                        break;
+                      }
                   }
                 q++;
               }
-            if (!SyncImagePixelsEx(implode_image,exception))
-              thread_status=MagickFail;
+            if (thread_status != MagickFail)
+              if (!SyncImagePixelsEx(implode_image,exception))
+                thread_status=MagickFail;
           }
 #if defined(HAVE_OPENMP)
 #  pragma omp critical (GM_ImplodeImage)
@@ -957,7 +962,7 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
 
   unsigned long
     row_count=0;
-  
+
   MagickPassFail
     status=MagickPass;
 
@@ -994,10 +999,10 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       const PixelPacket
         *p,
         *r;
-    
+
       PixelPacket
         *q;
-    
+
       long
         x;
 
@@ -1060,7 +1065,7 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
                         intensity=ru->red;
                       else
                         intensity=PixelIntensityToQuantum(ru);
-                        
+
                       hp=histogram+ScaleQuantumToChar(intensity);
                       (*hp)++;
                       if (*hp > count)
@@ -1086,7 +1091,7 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
           if (!MagickMonitorFormatted(row_count,image->rows,exception,
                                       OilPaintImageText,image->filename))
             thread_status=MagickFail;
-          
+
         if (thread_status == MagickFail)
           status=MagickFail;
       }
@@ -1138,7 +1143,7 @@ SolarizeImagePixelsCB(void *mutable_data,         /* User provided mutable data
     threshold = *((const double *) immutable_data);
 
   register long
-    i;  
+    i;
 
   ARG_NOT_USED(mutable_data);
   ARG_NOT_USED(image);
@@ -1534,7 +1539,7 @@ MagickExport Image *SwirlImage(const Image *image,double degrees,
       {
         register PixelPacket
           *q;
-    
+
         register long
           x;
 
@@ -1578,22 +1583,27 @@ MagickExport Image *SwirlImage(const Image *image,double degrees,
                       cosine,
                       factor,
                       sine;
-            
+
                     /*
                       Swirl the pixel.
                     */
                     factor=1.0-sqrt(distance)/radius;
                     sine=sin(degrees*factor*factor);
                     cosine=cos(degrees*factor*factor);
-                    InterpolateViewColor(image_view,q,
-                                         (cosine*x_distance-sine*y_distance)/x_scale+x_center,
-                                         (sine*x_distance+cosine*y_distance)/y_scale+y_center,
-                                         exception);
+                    if (InterpolateViewColor(image_view,q,
+                                             (cosine*x_distance-sine*y_distance)/x_scale+x_center,
+                                             (sine*x_distance+cosine*y_distance)/y_scale+y_center,
+                                             exception) == MagickFail)
+                      {
+                        thread_status=MagickFail;
+                        break;
+                      }
                   }
                 q++;
               }
-            if (!SyncImagePixelsEx(swirl_image,exception))
-              thread_status=MagickFail;
+            if (thread_status != MagickFail)
+              if (!SyncImagePixelsEx(swirl_image,exception))
+                thread_status=MagickFail;
           }
 #if defined(HAVE_OPENMP)
 #  pragma omp critical (GM_SwirlImage)
@@ -1651,7 +1661,7 @@ MagickExport Image *WaveImage(const Image *image,const double amplitude,
 
   VirtualPixelMethod
     virtual_pixel_method;
-    
+
   double
     *sine_map;
 
@@ -1747,7 +1757,7 @@ MagickExport Image *WaveImage(const Image *image,const double amplitude,
         thread_status=status;
         if (thread_status == MagickFail)
           continue;
-        
+
         image_view=AccessDefaultCacheView(image);
         q=SetImagePixelsEx(wave_image,0,y,wave_image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
@@ -1756,12 +1766,17 @@ MagickExport Image *WaveImage(const Image *image,const double amplitude,
           {
             for (x=0; x < (long) wave_image->columns; x++)
               {
-                InterpolateViewColor(image_view,&q[x],(double) x,
-                                     (double) y-sine_map[x],
-                                     exception);
+                if (InterpolateViewColor(image_view,&q[x],(double) x,
+                                         (double) y-sine_map[x],
+                                         exception) == MagickFail)
+                  {
+                    thread_status=MagickFail;
+                    break;
+                  }
               }
-            if (!SyncImagePixelsEx(wave_image,exception))
-              thread_status=MagickFail;
+            if (thread_status != MagickFail)
+              if (!SyncImagePixelsEx(wave_image,exception))
+                thread_status=MagickFail;
           }
 #if defined(HAVE_OPENMP)
 #  pragma omp critical (GM_WaveImage)
@@ -1772,7 +1787,7 @@ MagickExport Image *WaveImage(const Image *image,const double amplitude,
             if (!MagickMonitorFormatted(row_count,wave_image->rows,exception,
                                         WaveImageText,image->filename))
               thread_status=MagickFail;
-          
+
           if (thread_status == MagickFail)
             status=MagickFail;
         }
