@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2016 GraphicsMagick Group
+% Copyright (C) 2003 - 2017 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -368,7 +368,7 @@ DestroyModuleInfoEntry(ModuleInfo *entry)
       MagickFreeMemory(entry->magick);
       MagickFreeMemory(entry->name);
     }
-  MagickFreeMemory(entry);    
+  MagickFreeMemory(entry);
 }
 
 MagickExport void
@@ -465,7 +465,7 @@ DestroyModuleInfo(void)
 */
 MagickExport MagickPassFail
 ExecuteModuleProcess(const char *tag,Image **image,
-		     const int argc,char **argv)
+                     const int argc,char **argv)
 {
   ModuleHandle
     handle;
@@ -503,7 +503,7 @@ ExecuteModuleProcess(const char *tag,Image **image,
       {
         char
           message[MaxTextExtent];
-        
+
         FormatString(message,"\"%.256s: %.256s\"",module_path,lt_dlerror());
         ThrowException(&(*image)->exception,ModuleError,UnableToLoadModule,
           message);
@@ -646,7 +646,7 @@ GetCoderInfo(const char *tag, ExceptionInfo *exception)
 
 static MagickPassFail
 FindMagickModule(const char *filename,MagickModuleType module_type,
-		 char *path,ExceptionInfo *exception)
+                 char *path,ExceptionInfo *exception)
 {
   MagickMap
     path_map = (MagickMap) NULL;
@@ -664,7 +664,7 @@ FindMagickModule(const char *filename,MagickModuleType module_type,
   assert(path != (char *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
   (void) strlcpy(path,filename,MaxTextExtent);
-  
+
   if (InitializeModuleSearchPath(module_type,exception) == MagickFail)
     return (status);
 
@@ -688,15 +688,22 @@ FindMagickModule(const char *filename,MagickModuleType module_type,
         break;
       }
     }
-  
+
   path_map_iterator=MagickMapAllocateIterator(path_map);
-  
+  if (path_map_iterator == (MagickMapIterator) NULL)
+    {
+      path[0]='\0';
+      ThrowException(exception,ResourceLimitError,MemoryAllocationFailed,
+                     "MagickMapAllocateIterator");
+      return MagickFail;
+    }
+
   if (IsEventLogging())
     {
       char
         list_seperator[2],
         *search_path=0;
-      
+
       list_seperator[0]=DirectoryListSeparator;
       list_seperator[1]='\0';
       while(MagickMapIterateNext(path_map_iterator,&key))
@@ -706,21 +713,21 @@ FindMagickModule(const char *filename,MagickModuleType module_type,
           (void) ConcatenateString(&search_path,
                                    MagickMapDereferenceIterator(path_map_iterator,0));
         }
-      
+
       (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
                             "Searching for module file \"%s\" in path \"%s\"",
-			    filename,search_path);
-      
+                            filename,search_path);
+
       MagickFreeMemory(search_path);
       MagickMapIterateToFront(path_map_iterator);
     }
-  
+
   while(MagickMapIterateNext(path_map_iterator,&key))
     {
       FormatString(path,"%.1024s%.256s",
                    (const char *)MagickMapDereferenceIterator(path_map_iterator,0),
                    filename);
-      
+
       if (IsAccessible(path))
         {
           status=MagickPass;
@@ -763,9 +770,9 @@ FindMagickModule(const char *filename,MagickModuleType module_type,
 */
 static MagickPassFail
 GetModuleListForDirectory(const char *path,           /* Directory to scan. */
-			  char **list,                /* List to extend. */
-			  unsigned long *max_entries, /* Allocated list entries. */
-			  ExceptionInfo *exception )  /* Any exception. */
+                          char **list,                /* List to extend. */
+                          unsigned long *max_entries, /* Allocated list entries. */
+                          ExceptionInfo *exception )  /* Any exception. */
 {
   char
     module_tag[MaxTextExtent];
@@ -867,9 +874,6 @@ GetModuleList(ExceptionInfo *exception)
   unsigned long
     max_entries;
 
-  if (InitializeModuleSearchPath(MagickCoderModule,exception) == MagickFail)
-    return ((char **) NULL);
-
   max_entries=MAX_MODULES;
   modules=MagickAllocateMemory(char **,(max_entries+1)*sizeof(char *));
   if (modules == (char **) NULL)
@@ -968,7 +972,7 @@ InitializeMagickModules(void)
 */
 static void
 AddModulePath(MagickMap path_map, unsigned int *path_index,
-	      const char *path,ExceptionInfo *exception)
+              const char *path,ExceptionInfo *exception)
 {
   char
     key[MaxTextExtent];
@@ -1004,7 +1008,7 @@ ChopPathComponents(char *path,const unsigned long components)
 
 MagickPassFail
 InitializeModuleSearchPath(MagickModuleType module_type,
-			   ExceptionInfo *exception)
+                           ExceptionInfo *exception)
 {
   MagickMap
     path_map = (MagickMap) NULL;
@@ -1066,19 +1070,22 @@ InitializeModuleSearchPath(MagickModuleType module_type,
       const char
         *end = NULL,
         *start = module_path;
-      
+
       end=start+strlen(start);
       while ( start < end )
         {
           char
             buffer[MaxTextExtent];
-          
+
           const char
             *seperator;
-          
+
           int
             length;
-          
+
+          MagickBool
+            skip=MagickFalse;
+
           seperator = strchr(start,DirectoryListSeparator);
           if (seperator)
             length=seperator-start;
@@ -1088,9 +1095,52 @@ InitializeModuleSearchPath(MagickModuleType module_type,
             length = MaxTextExtent-1;
           (void) strncpy(buffer,start,length);
           buffer[length]='\0';
-          if (buffer[length-1] != DirectorySeparator[0])
-            (void) strcat(buffer,DirectorySeparator);
-          AddModulePath(path_map,&path_index,buffer,exception);
+#if HAVE_REALPATH
+          {
+            char
+              real_path[PATH_MAX+1];
+
+            if (realpath(buffer,real_path) != NULL)
+              {
+                length=strlcpy(buffer,real_path,sizeof(buffer));
+                if ((size_t) length >= sizeof(buffer))
+                  {
+                    /* Buffer overflow */
+                    (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                                          "Module path: Buffer overflow of "
+                                          "component \"%s\"",
+                                          real_path);
+                    length=0;
+                    skip=MagickTrue;
+                  }
+              }
+            else
+              {
+                /* Path does not exist */
+                (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                                      "Module path: Skipping invalid path "
+                                      "component \"%s\" (%s)",
+                                      buffer,strerror(errno));
+                skip=MagickTrue;
+              }
+          }
+#else /* !HAVE_REALPATH */
+          if (IsAccessibleNoLogging(buffer) != MagickTrue)
+            {
+              /* Path does not exist */
+              (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+                                    "Module path: Skipping invalid path "
+                                    "component \"%s\" (%s)",
+                                    buffer,strerror(errno));
+              skip=MagickTrue;
+            }
+#endif
+          if (skip == MagickFalse)
+            {
+              if (buffer[length-1] != DirectorySeparator[0])
+                (void) strcat(buffer,DirectorySeparator);
+              AddModulePath(path_map,&path_index,buffer,exception);
+            }
           start += length+1;
         }
     }
@@ -1284,30 +1334,30 @@ GetModuleInfo(const char *name,ExceptionInfo *exception)
   else
     {
       register ModuleInfo
-	*p;
+        *p;
 
       for (p=module_list; p != (ModuleInfo *) NULL; p=p->next)
-	if (LocaleCompare(p->name,name) == 0)
-	  break;
+        if (LocaleCompare(p->name,name) == 0)
+          break;
 
       if (p != (ModuleInfo *) NULL)
-	{
-	  module_info=p;
-	  if (p != module_list)
-	    {
-	      /*
-		Self-adjusting list.
-	      */
-	      if (p->previous != (ModuleInfo *) NULL)
-		p->previous->next=p->next;
-	      if (p->next != (ModuleInfo *) NULL)
-		p->next->previous=p->previous;
-	      p->previous=(ModuleInfo *) NULL;
-	      p->next=module_list;
-	      module_list->previous=p;
-	      module_list=p;
-	    }
-	}
+        {
+          module_info=p;
+          if (p != module_list)
+            {
+              /*
+                Self-adjusting list.
+              */
+              if (p->previous != (ModuleInfo *) NULL)
+                p->previous->next=p->next;
+              if (p->next != (ModuleInfo *) NULL)
+                p->next->previous=p->previous;
+              p->previous=(ModuleInfo *) NULL;
+              p->next=module_list;
+              module_list->previous=p;
+              module_list=p;
+            }
+        }
     }
   return (module_info);
 }
@@ -1638,8 +1688,8 @@ OpenModules(ExceptionInfo *exception)
 */
 static MagickPassFail
 ReadModuleConfigureFile(const char *basename,
-			const unsigned int depth,
-			ExceptionInfo *exception)
+                        const unsigned int depth,
+                        ExceptionInfo *exception)
 {
   size_t
     length;
@@ -1651,36 +1701,36 @@ ReadModuleConfigureFile(const char *basename,
   if (depth == 0)
     {
       size_t
-	i;
+        i;
 
       /*
-	Load default set of module aliases from the static aliases table.
+        Load default set of module aliases from the static aliases table.
       */
       for (i=0 ; i < sizeof(ModuleAliases)/sizeof(ModuleAliases[0]); i++)
-	{
-	  ModuleInfo
-	    *module_info;
+        {
+          ModuleInfo
+            *module_info;
 
-	  module_info=MagickAllocateMemory(ModuleInfo *,sizeof(ModuleInfo));
-	  if (module_info == (ModuleInfo *) NULL)
-	    MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
-			      UnableToAllocateModuleInfo);
-	  module_info->path=(char *) BuiltInPath;
-	  module_info->magick=(char *) ModuleAliases[i].magick;
-	  module_info->name=(char *) ModuleAliases[i].name;
-	  module_info->stealth=MagickFalse;
-	  module_info->signature=MagickSignature;
-	  module_info->previous=(ModuleInfo *) NULL;
-	  module_info->next=(ModuleInfo *) NULL;
-	  if (module_list == (ModuleInfo *) NULL)
-	    {
-	      module_list=module_info;
-	      continue;
-	    }
-	  module_list->next=module_info;
-	  module_info->previous=module_list;
-	  module_list=module_list->next;
-	}
+          module_info=MagickAllocateMemory(ModuleInfo *,sizeof(ModuleInfo));
+          if (module_info == (ModuleInfo *) NULL)
+            MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+                              UnableToAllocateModuleInfo);
+          module_info->path=(char *) BuiltInPath;
+          module_info->magick=(char *) ModuleAliases[i].magick;
+          module_info->name=(char *) ModuleAliases[i].name;
+          module_info->stealth=MagickFalse;
+          module_info->signature=MagickSignature;
+          module_info->previous=(ModuleInfo *) NULL;
+          module_info->next=(ModuleInfo *) NULL;
+          if (module_list == (ModuleInfo *) NULL)
+            {
+              module_list=module_info;
+              continue;
+            }
+          module_list->next=module_info;
+          module_info->previous=module_list;
+          module_list=module_list->next;
+        }
     }
 
   /*
@@ -1690,12 +1740,12 @@ ReadModuleConfigureFile(const char *basename,
   if (depth == 0)
     {
       ExceptionInfo
-	exception_local;
+        exception_local;
 
       GetExceptionInfo(&exception_local);
       xml=(char *) GetConfigureBlob(basename,path,&length,&exception_local);
       if (exception_local.severity != ConfigureError)
-	CopyException(exception,&exception_local);
+        CopyException(exception,&exception_local);
       DestroyExceptionInfo(&exception_local);
     }
   else
@@ -1705,12 +1755,12 @@ ReadModuleConfigureFile(const char *basename,
   if (xml != (char *) NULL)
     {
       char
-	keyword[MaxTextExtent],
-	*q,
-	*token;
+        keyword[MaxTextExtent],
+        *q,
+        *token;
 
       MagickBool
-	in_entry;
+        in_entry;
 
       size_t
         token_max_length;
@@ -1719,155 +1769,155 @@ ReadModuleConfigureFile(const char *basename,
       token_max_length=strlen(token);
       in_entry=MagickFalse;
       for (q=xml; *q != '\0'; )
-	{
-	  /*
-	    Interpret XML.
-	  */
-	  MagickGetToken(q,&q,token,token_max_length);
-	  if (*token == '\0')
-	    break;
-	  (void) strlcpy(keyword,token,sizeof(keyword));
-	  if (LocaleNCompare(keyword,"<!--",4) == 0)
-	    {
-	      /*
-		Comment element.
-	      */
-	      while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-		MagickGetToken(q,&q,token,token_max_length);
-	      continue;
-	    }
-	  if (LocaleCompare(keyword,"<include") == 0)
-	    {
-	      /*
-		Include element.
-	      */
-	      while ((*token != '>') && (*q != '\0'))
-		{
-		  (void) strlcpy(keyword,token,MaxTextExtent);
-		  MagickGetToken(q,&q,token,token_max_length);
-		  if (*token != '=')
-		    continue;
-		  MagickGetToken(q,&q,token,token_max_length);
-		  if (LocaleCompare(keyword,"file") == 0)
-		    {
-		      if (depth > 200)
-			ThrowException(exception,ConfigureError,
-				       IncludeElementNestedTooDeeply,path);
-		      else
-			{
-			  char
-			    filename[MaxTextExtent];
+        {
+          /*
+            Interpret XML.
+          */
+          MagickGetToken(q,&q,token,token_max_length);
+          if (*token == '\0')
+            break;
+          (void) strlcpy(keyword,token,sizeof(keyword));
+          if (LocaleNCompare(keyword,"<!--",4) == 0)
+            {
+              /*
+                Comment element.
+              */
+              while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
+                MagickGetToken(q,&q,token,token_max_length);
+              continue;
+            }
+          if (LocaleCompare(keyword,"<include") == 0)
+            {
+              /*
+                Include element.
+              */
+              while ((*token != '>') && (*q != '\0'))
+                {
+                  (void) strlcpy(keyword,token,MaxTextExtent);
+                  MagickGetToken(q,&q,token,token_max_length);
+                  if (*token != '=')
+                    continue;
+                  MagickGetToken(q,&q,token,token_max_length);
+                  if (LocaleCompare(keyword,"file") == 0)
+                    {
+                      if (depth > 200)
+                        ThrowException(exception,ConfigureError,
+                                       IncludeElementNestedTooDeeply,path);
+                      else
+                        {
+                          char
+                            filename[MaxTextExtent];
 
-			  GetPathComponent(path,HeadPath,filename);
-			  if (*filename != '\0')
-			    (void) strlcat(filename,DirectorySeparator,sizeof(filename));
-			  (void) strlcat(filename,token,sizeof(filename));
-			  (void) ReadModuleConfigureFile(filename,depth+1,exception);
-			}
-		      if (module_list != (ModuleInfo *) NULL)
-			while (module_list->next != (ModuleInfo *) NULL)
-			  module_list=module_list->next;
-		    }
-		}
-	      continue;
-	    }
-	  if (LocaleCompare(keyword,"<module") == 0)
-	    {
-	      ModuleInfo
-		*module_info;
+                          GetPathComponent(path,HeadPath,filename);
+                          if (*filename != '\0')
+                            (void) strlcat(filename,DirectorySeparator,sizeof(filename));
+                          (void) strlcat(filename,token,sizeof(filename));
+                          (void) ReadModuleConfigureFile(filename,depth+1,exception);
+                        }
+                      if (module_list != (ModuleInfo *) NULL)
+                        while (module_list->next != (ModuleInfo *) NULL)
+                          module_list=module_list->next;
+                    }
+                }
+              continue;
+            }
+          if (LocaleCompare(keyword,"<module") == 0)
+            {
+              ModuleInfo
+                *module_info;
 
-	      /*
-		Allocate memory for the module list.
-	      */
-	      in_entry=MagickTrue;
-	      module_info=MagickAllocateMemory(ModuleInfo *,sizeof(ModuleInfo));
-	      if (module_info == (ModuleInfo *) NULL)
-		MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
-				  UnableToAllocateModuleInfo);
-	      (void) memset(module_info,0,sizeof(ModuleInfo));
-	      module_info->path=AcquireString(path);
-	      module_info->signature=MagickSignature;
-	      if (module_list == (ModuleInfo *) NULL)
-		{
-		  module_list=module_info;
-		  continue;
-		}
-	      module_list->next=module_info;
-	      module_info->previous=module_list;
-	      module_list=module_list->next;
-	      continue;
-	    }
-	  if (LocaleCompare(keyword,"/>") == 0)
-	    {
-	      /*
-		Closing a module alias specification.
-	      */
-	      if (in_entry)
-		{
-		  /*
-		    Remove any existing entry with same name (last one wins).
-		  */
-		  {
-		    ModuleInfo
-		      *module_info;
-	      
-		    for (module_info=module_list->previous;
-			 module_info != (ModuleInfo *) NULL;
-			 module_info=module_info->previous)
-		      {
-			if (LocaleCompare(module_list->magick,module_info->magick) == 0)
-			  {
-			    DestroyModuleInfoEntry(module_info);
-			    break;
-			  }
-		      }
-		  }
-		  in_entry=MagickFalse;
-		}
-	    }
-	  if (module_list == (ModuleInfo *) NULL)
-	    continue;
-	  MagickGetToken(q,(char **) NULL,token,token_max_length);
-	  if (*token != '=')
-	    continue;
-	  MagickGetToken(q,&q,token,token_max_length);
-	  MagickGetToken(q,&q,token,token_max_length);
-	  switch (*keyword)
-	    {
-	    case 'M':
-	    case 'm':
-	      {
-		if (LocaleCompare((char *) keyword,"magick") == 0)
-		  {
-		    module_list->magick=AcquireString(token);
-		    break;
-		  }
-		break;
-	      }
-	    case 'N':
-	    case 'n':
-	      {
-		if (LocaleCompare((char *) keyword,"name") == 0)
-		  {
-		    module_list->name=AcquireString(token);
-		    break;
-		  }
-		break;
-	      }
-	    case 'S':
-	    case 's':
-	      {
-		if (LocaleCompare((char *) keyword,"stealth") == 0)
-		  {
-		    module_list->stealth=LocaleCompare(token,"True") == 0;
-		    break;
-		  }
-		break;
-	      }
-	    default:
-	      break;
-	    }
-	}
+              /*
+                Allocate memory for the module list.
+              */
+              in_entry=MagickTrue;
+              module_info=MagickAllocateMemory(ModuleInfo *,sizeof(ModuleInfo));
+              if (module_info == (ModuleInfo *) NULL)
+                MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+                                  UnableToAllocateModuleInfo);
+              (void) memset(module_info,0,sizeof(ModuleInfo));
+              module_info->path=AcquireString(path);
+              module_info->signature=MagickSignature;
+              if (module_list == (ModuleInfo *) NULL)
+                {
+                  module_list=module_info;
+                  continue;
+                }
+              module_list->next=module_info;
+              module_info->previous=module_list;
+              module_list=module_list->next;
+              continue;
+            }
+          if (LocaleCompare(keyword,"/>") == 0)
+            {
+              /*
+                Closing a module alias specification.
+              */
+              if (in_entry)
+                {
+                  /*
+                    Remove any existing entry with same name (last one wins).
+                  */
+                  {
+                    ModuleInfo
+                      *module_info;
+
+                    for (module_info=module_list->previous;
+                         module_info != (ModuleInfo *) NULL;
+                         module_info=module_info->previous)
+                      {
+                        if (LocaleCompare(module_list->magick,module_info->magick) == 0)
+                          {
+                            DestroyModuleInfoEntry(module_info);
+                            break;
+                          }
+                      }
+                  }
+                  in_entry=MagickFalse;
+                }
+            }
+          if (module_list == (ModuleInfo *) NULL)
+            continue;
+          MagickGetToken(q,(char **) NULL,token,token_max_length);
+          if (*token != '=')
+            continue;
+          MagickGetToken(q,&q,token,token_max_length);
+          MagickGetToken(q,&q,token,token_max_length);
+          switch (*keyword)
+            {
+            case 'M':
+            case 'm':
+              {
+                if (LocaleCompare((char *) keyword,"magick") == 0)
+                  {
+                    module_list->magick=AcquireString(token);
+                    break;
+                  }
+                break;
+              }
+            case 'N':
+            case 'n':
+              {
+                if (LocaleCompare((char *) keyword,"name") == 0)
+                  {
+                    module_list->name=AcquireString(token);
+                    break;
+                  }
+                break;
+              }
+            case 'S':
+            case 's':
+              {
+                if (LocaleCompare((char *) keyword,"stealth") == 0)
+                  {
+                    module_list->stealth=LocaleCompare(token,"True") == 0;
+                    break;
+                  }
+                break;
+              }
+            default:
+              break;
+            }
+        }
       MagickFreeMemory(token);
       MagickFreeMemory(xml);
     }
